aboutsummaryrefslogtreecommitdiffstats
path: root/activemodel
diff options
context:
space:
mode:
authorPrem Sichanugrist <s@sikachu.com>2011-04-10 15:39:01 +0800
committerJosé Valim <jose.valim@gmail.com>2011-04-10 18:49:28 +0800
commitf6540211b5b9133c9f93c11655a04d613c237e67 (patch)
treefda530dfc1f19d834bd1a638f55366be267b76da /activemodel
parent58594be680e8712c9e7352f184e15972d02cd4af (diff)
downloadrails-f6540211b5b9133c9f93c11655a04d613c237e67.tar.gz
rails-f6540211b5b9133c9f93c11655a04d613c237e67.tar.bz2
rails-f6540211b5b9133c9f93c11655a04d613c237e67.zip
Add :use_include option to allow user to explicitly use `Range#include?` method in Ruby 1.9
In Ruby 1.9 we're currently use `Range#cover?` to fix the performance problem. However, there might be the case that you want to use `Range#include?` instead. This patch will give you that option.
Diffstat (limited to 'activemodel')
-rw-r--r--activemodel/lib/active_model/validations/exclusion.rb16
-rw-r--r--activemodel/lib/active_model/validations/inclusion.rb16
-rw-r--r--activemodel/test/cases/validations/exclusion_validation_test.rb10
-rw-r--r--activemodel/test/cases/validations/inclusion_validation_test.rb9
4 files changed, 37 insertions, 14 deletions
diff --git a/activemodel/test/cases/validations/exclusion_validation_test.rb b/activemodel/test/cases/validations/exclusion_validation_test.rb
index fe0d13ec33..e1807a3f34 100644
--- a/activemodel/test/cases/validations/exclusion_validation_test.rb
+++ b/activemodel/test/cases/validations/exclusion_validation_test.rb
@@ -63,4 +63,14 @@ class ExclusionValidationTest < ActiveModel::TestCase
p.author_name = "sikachu"
assert_raise(ArgumentError){ p.valid? }
end
+
+ def test_validates_inclusion_with_explicit_include
+ range = (1..100)
+ Topic.validates_exclusion_of :title, :in => range, :use_include => true
+ range.expects(:include?).returns(false)
+
+ t = Topic.new
+ t.title = 102
+ assert t.valid?
+ end
end
diff --git a/activemodel/test/cases/validations/inclusion_validation_test.rb b/activemodel/test/cases/validations/inclusion_validation_test.rb
index e6adb6d187..d183b42c2b 100644
--- a/activemodel/test/cases/validations/inclusion_validation_test.rb
+++ b/activemodel/test/cases/validations/inclusion_validation_test.rb
@@ -95,4 +95,13 @@ class InclusionValidationTest < ActiveModel::TestCase
p.author_name = "sikachu"
assert_raise(ArgumentError){ p.valid? }
end
+
+ def test_validates_inclusion_with_explicit_include
+ range = (1..100)
+ Topic.validates_inclusion_of :title, :in => range, :use_include => true
+
+ t = Topic.new
+ t.title = 42
+ assert t.valid?
+ end
end
diff --git a/activemodel/lib/active_model/validations/exclusion.rb b/activemodel/lib/active_model/validations/exclusion.rb
index abc1bfae78..e8d7bb162a 100644
--- a/activemodel/lib/active_model/validations/exclusion.rb
+++ b/activemodel/lib/active_model/validations/exclusion.rb
@@ -16,8 +16,8 @@ module ActiveModel
def validate_each(record, attribute, value)
exclusions = options[:in].respond_to?(:call) ? options[:in].call(record) : options[:in]
- if exclusions.send(inclusion_method(exclusions), value)
- record.errors.add(attribute, :exclusion, options.except(:in).merge!(:value => value))
+ if exclusions.send(inclusion_method(exclusions, options[:use_include]), value)
+ record.errors.add(attribute, :exclusion, options.except(:in, :use_include).merge!(:value => value))
end
rescue NoMethodError
raise ArgumentError, "Exclusion validation for :#{attribute} in #{record.class.name}: #{ERROR_MESSAGE}"
@@ -28,8 +28,8 @@ module ActiveModel
# In Ruby 1.9 <tt>Range#include?</tt> on non-numeric ranges checks all possible values in the
# range for equality, so it may be slow for large ranges. The new <tt>Range#cover?</tt>
# uses the previous logic of comparing a value with the range endpoints.
- def inclusion_method(enumerable)
- enumerable.is_a?(Range) ? :cover? : :include?
+ def inclusion_method(enumerable, use_include = nil)
+ !use_include && enumerable.is_a?(Range) ? :cover? : :include?
end
end
@@ -45,9 +45,11 @@ module ActiveModel
#
# Configuration options:
# * <tt>:in</tt> - An enumerable object of items that the value shouldn't be part of.
# This can be supplied as a proc or lambda which returns an enumerable. If the enumerable
# is a range the test is performed with <tt>Range#cover?</tt>
# (backported in Active Support for 1.8), otherwise with <tt>include?</tt>.
# * <tt>:message</tt> - Specifies a custom error message (default is: "is reserved").
# * <tt>:allow_nil</tt> - If set to true, skips this validation if the attribute is +nil+ (default is +false+).
# * <tt>:allow_blank</tt> - If set to true, skips this validation if the attribute is blank (default is +false+).
@@ -45,9 +45,11 @@ module ActiveModel
# Configuration options:
# * <tt>:in</tt> - An enumerable object of items that the value shouldn't be part of.
- # This can be supplied as a proc or lambda which returns an enumerable. If the enumerable
- # is a range the test is performed with <tt>Range#cover?</tt>
- # (backported in Active Support for 1.8), otherwise with <tt>include?</tt>.
+ # This can be supplied as a proc or lambda which returns an enumerable.
+ # * <tt>:use_include</tt> - If set to true and the enumerable in <tt>:in</tt> option is a range,
+ # it will explicitly use <tt>Range#include?</tt> to perform the test. Otherwise <tt>Range#cover?</tt>
+ # will be used to perform the test for performance reason.
+ # (Range#cover? was backported in Active Support for 1.8.x)
# * <tt>:message</tt> - Specifies a custom error message (default is: "is reserved").
# * <tt>:allow_nil</tt> - If set to true, skips this validation if the attribute is +nil+ (default is +false+).
diff --git a/activemodel/lib/active_model/validations/inclusion.rb b/activemodel/lib/active_model/validations/inclusion.rb
index cb46547e92..2b14f84bb5 100644
--- a/activemodel/lib/active_model/validations/inclusion.rb
+++ b/activemodel/lib/active_model/validations/inclusion.rb
@@ -16,8 +16,8 @@ module ActiveModel
def validate_each(record, attribute, value)
exclusions = options[:in].respond_to?(:call) ? options[:in].call(record) : options[:in]
- unless exclusions.send(inclusion_method(exclusions), value)
- record.errors.add(attribute, :inclusion, options.except(:in).merge!(:value => value))
+ unless exclusions.send(inclusion_method(exclusions, options[:use_include]), value)
+ record.errors.add(attribute, :inclusion, options.except(:in, :use_include).merge!(:value => value))
end
rescue NoMethodError
raise ArgumentError, "Exclusion validation for :#{attribute} in #{record.class.name}: #{ERROR_MESSAGE}"
@@ -28,8 +28,8 @@ module ActiveModel
# In Ruby 1.9 <tt>Range#include?</tt> on non-numeric ranges checks all possible values in the
# range for equality, so it may be slow for large ranges. The new <tt>Range#cover?</tt>
# uses the previous logic of comparing a value with the range endpoints.
- def inclusion_method(enumerable)
- enumerable.is_a?(Range) ? :cover? : :include?
+ def inclusion_method(enumerable, use_include = nil)
+ !use_include && enumerable.is_a?(Range) ? :cover? : :include?
end
end
@@ -45,9 +45,11 @@ module ActiveModel
#
# Configuration options:
# * <tt>:in</tt> - An enumerable object of available items. This can be
# supplied as a proc or lambda which returns an enumerable. If the enumerable
# is a range the test is performed with <tt>Range#cover?</tt>
# (backported in Active Support for 1.8), otherwise with <tt>include?</tt>.
# * <tt>:message</tt> - Specifies a custom error message (default is: "is not included in the list").
# * <tt>:allow_nil</tt> - If set to true, skips this validation if the attribute is +nil+ (default is +false+).
# * <tt>:allow_blank</tt> - If set to true, skips this validation if the attribute is blank (default is +false+).
@@ -45,9 +45,11 @@ module ActiveModel
# Configuration options:
# * <tt>:in</tt> - An enumerable object of available items. This can be
- # supplied as a proc or lambda which returns an enumerable. If the enumerable
- # is a range the test is performed with <tt>Range#cover?</tt>
- # (backported in Active Support for 1.8), otherwise with <tt>include?</tt>.
+ # supplied as a proc or lambda which returns an enumerable.
+ # * <tt>:use_include</tt> - If set to true and the enumerable in <tt>:in</tt> option is a range,
+ # it will explicitly use <tt>Range#include?</tt> to perform the test. Otherwise <tt>Range#cover?</tt>
+ # will be used to perform the test for performance reason.
+ # (Range#cover? was backported in Active Support for 1.8.x)
# * <tt>:message</tt> - Specifies a custom error message (default is: "is not included in the list").
# * <tt>:allow_nil</tt> - If set to true, skips this validation if the attribute is +nil+ (default is +false+).